<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xalan"
                xmlns:lav="http://software.in2p3.fr/lavoisier/config.xsd"
                xmlns="http://software.in2p3.fr/lavoisier/config.xsd"
                exclude-result-prefixes="xalan lav">
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4"/>
    <xsl:strip-space elements="lav:*"/>
    <xsl:param name="validator.Disable">false</xsl:param>

    <!-- checks -->
    <xsl:template match="/">
        <xsl:for-each select="descendant::lav:parameter[contains(@eval,'post()')][ancestor::lav:insert | ancestor::lav:replace]">
            <xsl:message terminate="yes">Error in view '<xsl:value-of select="ancestor::lav:view/@name"/>':
                Function post() not allowed in elements insert and replace</xsl:message>
        </xsl:for-each>
        <xsl:for-each select="descendant::lav:view[lav:cache]/lav:argument">
            <xsl:for-each select="following-sibling::lav:*/descendant-or-self::lav:*
                                  [not(self::lav:argument | self::lav:variable | ancestor::lav:cache | ancestor::lav:renderers)]
                                  /@*[contains(.,concat('$',current()/@name))]">
                <xsl:message terminate="yes">Error in view '<xsl:value-of select="parent::lav:view/@name"/>':
                    Detected incompatibility between cache and this XPath expression: <xsl:value-of select="."/>
                </xsl:message>
            </xsl:for-each>
        </xsl:for-each>
        <xsl:apply-templates/>
    </xsl:template>

    <!-- add default post-processors -->
    <xsl:template match="lav:config">
        <xsl:copy><xsl:copy-of select="@*"/>
            <xsl:if test="not(lav:processors[not(@name) or @name='_post-view_'])">
                <processors/>
            </xsl:if>
            <xsl:if test="not(lav:processors[@name='disable-default-processors'])">
                <processors name="disable-default-processors"/>
            </xsl:if>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>

    <!-- replace default post-processors -->
    <xsl:template match="lav:view[lav:argument[not(@path-format = 'none')]]/@name">
        <xsl:copy-of select="."/>
        <xsl:if test="not(parent::lav:view/@post-processors)">
            <xsl:attribute name="post-processors">disable-default-processors</xsl:attribute>
        </xsl:if>
    </xsl:template>

    <!-- add default renderer -->
    <xsl:template match="lav:renderers[@html]">
        <xsl:copy>
            <xsl:apply-templates select="lav:renderer"/>
            <xsl:if test="not(lav:renderer[@type='DefaultRenderer'])">
                <renderer type="DefaultRenderer">
                    <xsl:apply-templates select="self::lav:renderers" mode="PARAMETER">
                        <xsl:with-param name="base">/resource/</xsl:with-param>
                    </xsl:apply-templates>
                </renderer>
            </xsl:if>
            <xsl:if test="not(lav:renderer[@type='HTMLRenderer'])">
                <renderer type="HTMLRenderer">
                    <xsl:apply-templates select="self::lav:renderers" mode="PARAMETER"/>
                </renderer>
            </xsl:if>
            <xsl:if test="not(lav:renderer[@type='PDFRenderer'])">
                <renderer type="PDFRenderer">
                    <xsl:apply-templates select="self::lav:renderers" mode="PARAMETER"/>
                </renderer>
            </xsl:if>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="lav:renderers[@html]/lav:renderer[(@type='DefaultRenderer' or @type='HTMLRenderer' or @type='PDFRenderer') and not(lav:parameter[@name='stylesheet'])]">
        <xsl:copy><xsl:apply-templates select="@*"/>
            <xsl:apply-templates/>
            <xsl:choose>
                <xsl:when test="@type='DefaultRenderer'">
                    <xsl:apply-templates select="parent::lav:renderers" mode="PARAMETER">
                        <xsl:with-param name="base">/resource/</xsl:with-param>
                    </xsl:apply-templates>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:apply-templates select="parent::lav:renderers" mode="PARAMETER"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="lav:renderers[@html]" mode="PARAMETER">
        <xsl:param name="base"/>
        <parameter name="stylesheet">
            <xsl:value-of select="$base"/><xsl:value-of select="@html"/>
        </parameter>
        <parameter name="parameters">
            <xsl:for-each select="lav:parameter">
                <entry key="{@name}">
                    <xsl:copy-of select="@eval | text()"/>
                </entry>
            </xsl:for-each>
        </parameter>
    </xsl:template>

    <!-- disable XML validation -->
    <xsl:template match="lav:processor[contains(@type, 'Validator') and $validator.Disable='true']"/>

    <!-- add default indexed cache parameter -->
    <xsl:template match="lav:cache[@type='IndexedFileCache' and not(lav:parameter/@name='xpath')]">
        <xsl:copy><xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="lav:parameter"/>
            <xsl:comment> generated automatically </xsl:comment>
            <parameter name="xpath" eval="path(false())"/>
            <xsl:apply-templates select="lav:trigger"/>
        </xsl:copy>
    </xsl:template>

    <!-- default rules -->
    <xsl:template match="@*">
        <xsl:copy-of select="."/>
    </xsl:template>
    <xsl:template match="*">
        <xsl:copy>
            <xsl:apply-templates select="@*|*|text()|comment()"/>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>