<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xalan"
                xmlns:lav="http://software.in2p3.fr/lavoisier/config.xsd"
                xmlns="http://software.in2p3.fr/lavoisier/config.xsd"
                exclude-result-prefixes="xalan lav">
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4"/>
    <xsl:strip-space elements="lav:*"/>

    <xsl:template match="lav:cache">
        <xsl:copy><xsl:copy-of select="@*[local-name()!='disabled']"/>
            <xsl:attribute name="disabled">
                <xsl:text>arguments()[@key='_stop_at_']/text()</xsl:text>
                <xsl:text> and arguments()[@key='view']/text()=view_id()</xsl:text>
                <xsl:if test="@disabled"> or (<xsl:value-of select="@disabled"/>)</xsl:if>
            </xsl:attribute>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="lav:processors/lav:processor">
        <xsl:copy><xsl:copy-of select="@*[local-name()!='disabled']"/>
            <xsl:attribute name="disabled">
                <xsl:text>arguments()[@key='_stop_at_']/text()</xsl:text>
                <xsl:if test="ancestor::lav:view"> &lt; <xsl:value-of select="count(preceding-sibling::lav:*)+1"/></xsl:if>
                <xsl:text> and arguments()[@key='view']/text()=view_id()</xsl:text>
                <xsl:if test="@disabled"> or (<xsl:value-of select="@disabled"/>)</xsl:if>
            </xsl:attribute>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="lav:processors/lav:*[not(self::lav:processor)]">
        <processor>
            <xsl:attribute name="disabled">
                <xsl:text>arguments()[@key='_stop_at_']/text()</xsl:text>
                <xsl:if test="ancestor::lav:view"> &lt; <xsl:value-of select="count(preceding-sibling::lav:*)+1"/></xsl:if>
                <xsl:text> and arguments()[@key='view']/text()=view_id()</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="type">
                <xsl:choose>
                    <xsl:when test="self::lav:aggregate">AppendAggregateProcessor</xsl:when>
                    <xsl:when test="self::lav:insert-parent">InsertParentProcessor</xsl:when>
                    <xsl:when test="self::lav:insert">InsertProcessor</xsl:when>
                    <xsl:when test="self::lav:merge and @count">MultiMergeProcessor</xsl:when>
                    <xsl:when test="self::lav:merge">MergeProcessor</xsl:when>
                    <xsl:when test="self::lav:move">MoveProcessor</xsl:when>
                    <xsl:when test="self::lav:copy">CopyProcessor</xsl:when>
                    <xsl:when test="self::lav:remove">RemoveProcessor</xsl:when>
                    <xsl:when test="self::lav:rename">RenameProcessor</xsl:when>
                    <xsl:when test="self::lav:replace">ReplaceProcessor</xsl:when>
                    <xsl:when test="self::lav:select and lav:group">SelectGroupByProcessor</xsl:when>
                    <xsl:when test="self::lav:select and lav:distinct">SelectDistinctProcessor</xsl:when>
                    <xsl:when test="self::lav:select and not(lav:*)">SelectProcessor</xsl:when>
                    <xsl:otherwise><xsl:message terminate="yes">Processor not supported: <xsl:value-of
                            select="local-name()"/></xsl:message></xsl:otherwise>
                </xsl:choose>
            </xsl:attribute>
            <xsl:attribute name="match">
                <xsl:apply-templates mode="match" select="@match"/>
            </xsl:attribute>
            <xsl:copy-of select="ancestor::lav:view/lav:namespace"/>
            <xsl:apply-templates mode="pixtl" select="@* | * | */@*"/>
        </processor>
    </xsl:template>

    <xsl:template mode="match" match="lav:move/@match | lav:copy/@match">
        <xsl:variable name="match">
            <xsl:call-template name="REMOVE_PREDICATE">
                <xsl:with-param name="path" select="."/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:value-of select="."/>
        <xsl:if test="not(contains($match,'/@') or contains($match,'/attribute::')
                       or contains($match,'/text()') or contains($match,'/comment()'))">[* or not(*)]</xsl:if>
    </xsl:template>
    <xsl:template mode="match" match="lav:aggregate/@match">
        <xsl:value-of select="."/> | <xsl:value-of select="."/>/<xsl:value-of select="parent::lav:aggregate/lav:to-last-child/@values"/>
    </xsl:template>
    <xsl:template mode="match" match="lav:select[lav:group]/@match">
        <xsl:text>/* | </xsl:text>
        <xsl:value-of select="."/>/<xsl:value-of select="parent::lav:select/lav:group/@by"/>
    </xsl:template>
    <xsl:template mode="match" match="lav:*/@match" priority="-1">
        <xsl:value-of select="."/>
    </xsl:template>

    <xsl:template mode="pixtl" match="lav:to-following-sibling">
        <parameter name="destination_axis">following-sibling</parameter>
    </xsl:template>
    <xsl:template mode="pixtl" match="lav:to-ancestor">
        <parameter name="destination_axis">ancestor</parameter>
    </xsl:template>
    <xsl:template mode="pixtl" match="lav:group">
        <xsl:variable name="ancestor">
            <xsl:call-template name="REVERT_PATH">
                <xsl:with-param name="path">
                    <xsl:call-template name="REMOVE_PREDICATE">
                        <xsl:with-param name="path" select="@by"/>
                    </xsl:call-template>
                </xsl:with-param>
            </xsl:call-template>
        </xsl:variable>
        <parameter name="ancestor" eval="{concat('.',$ancestor,'/parent::*')}"/>
    </xsl:template>
    <xsl:template mode="pixtl" match="lav:distinct"/>
    <xsl:template mode="pixtl" match="lav:to-last-child"/>

    <xsl:template mode="pixtl" match="lav:*/@match"/>
    <xsl:template mode="pixtl" match="lav:to-last-child/@values"/>
    <xsl:template mode="pixtl" match="lav:group/@by"/>

    <xsl:template mode="pixtl" match="@nodes | lav:to-following-sibling/@predicate | lav:to-ancestor/@predicate">
        <parameter>
            <xsl:attribute name="name"><xsl:apply-templates mode="name" select="."/></xsl:attribute>
            <xsl:attribute name="eval"><xsl:value-of select="."/></xsl:attribute>
        </parameter>
    </xsl:template>
    <xsl:template mode="pixtl" match="@*">
        <parameter>
            <xsl:attribute name="name"><xsl:apply-templates mode="name" select="."/></xsl:attribute>
            <xsl:value-of select="."/>
        </parameter>
    </xsl:template>

    <xsl:template mode="name" match="lav:aggregate/@function">function</xsl:template>
    <xsl:template mode="name" match="lav:group/@sort | lav:distinct/@sort">sort_type</xsl:template>
    <xsl:template mode="name" match="lav:group/@descending | lav:distinct/@descending">sort_descending</xsl:template>
    <xsl:template mode="name" match="lav:select/@depth | lav:remove/@depth">depth</xsl:template>
    <xsl:template mode="name" match="lav:select/@single">single_node</xsl:template>
    <xsl:template mode="name" match="lav:merge/@count">count</xsl:template>

    <xsl:template mode="name" match="lav:insert-parent/@name | lav:rename/@name | lav:to-last-child/@name">node_name</xsl:template>
    <xsl:template mode="name" match="lav:insert/@nodes | lav:replace/@nodes">nodes</xsl:template>
    <xsl:template mode="name" match="lav:to-following-sibling/@as | lav:to-ancestor/@as | lav:insert/@as">destination_as</xsl:template>
    <xsl:template mode="name" match="lav:to-following-sibling/@name | lav:to-ancestor/@name">destination_name</xsl:template>
    <xsl:template mode="name" match="lav:to-following-sibling/@predicate | lav:to-ancestor/@predicate">destination_predicate</xsl:template>
    <xsl:template mode="name" match="@*">
        <xsl:message terminate="yes">Unsupported attribute name: <xsl:value-of select="local-name()"/></xsl:message>
    </xsl:template>

    <xsl:template name="REMOVE_PREDICATE">
        <xsl:param name="path"/>
        <xsl:choose>
            <xsl:when test="contains($path, '[')">
                <xsl:call-template name="REMOVE_PREDICATE">
                    <xsl:with-param name="path" select="concat(substring-before($path,'['), substring-after($path,']'))"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$path"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template name="REVERT_PATH">
        <xsl:param name="path"/>
        <xsl:if test="contains($path, '/')">
            <xsl:call-template name="REVERT_PATH">
                <xsl:with-param name="path" select="substring-after($path, '/')"/>
            </xsl:call-template>/parent::<xsl:value-of select="substring-before($path, '/')"/>
        </xsl:if>
    </xsl:template>

    <!-- default rules -->
    <xsl:template match="@*">
        <xsl:copy-of select="."/>
    </xsl:template>
    <xsl:template match="*">
        <xsl:copy>
            <xsl:apply-templates select="@*|*|text()|comment()"/>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>