<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:x="http://www.w3.org/TR/xpath">
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4" xmlns:xalan="http://xml.apache.org/xslt"/>
    <xsl:strip-space elements="*"/>
    <xsl:variable name="foreignKeys" select="document('foreign-keys.xml')/schema/foreign-key"/>
    <xsl:variable name="lower">abcdefghijklmnopqrstuvwxyz</xsl:variable>
    <xsl:variable name="UPPER">ABCDEFGHIJKLMNOPQRSTUVWXYZ</xsl:variable>

    <xsl:template match="x:NameStep">
        <!-- add missing NameStep -->
        <xsl:if test="not(preceding-sibling::x:NameStep) and ancestor::x:NameStep[1][@prefix!=current()/@prefix]">
            <x:NameStep prefix="{ancestor::x:NameStep[1][@prefix!=current()/@prefix]/@prefix}" localName="?">
                <xsl:variable name="PRECEDING" select="ancestor::x:NameStep[1][@prefix!=current()/@prefix]"/>
                <xsl:variable name="PREC_TABLE" select="translate($PRECEDING/@prefix, $lower, $UPPER)"/>
                <xsl:variable name="PREC_COLUMN" select="translate($PRECEDING/@localName, $lower, $UPPER)"/>
                <xsl:variable name="TABLE" select="translate(@prefix, $lower, $UPPER)"/>
                <xsl:variable name="COLUMN" select="translate(@linkWithPreceding, $lower, $UPPER)"/>
                <xsl:variable name="FK1" select="$foreignKeys[@table=$PREC_TABLE and @column=$PREC_COLUMN]/references[@table=$TABLE]"/>
                <xsl:variable name="FK2" select="$foreignKeys[@table=$TABLE and @column=$COLUMN and references[@table=$PREC_TABLE]]"/>
                <xsl:choose>
                    <xsl:when test="$FK1">
                        <xsl:copy-of select="@localName"/>
                    </xsl:when>
                    <xsl:when test="$FK2">
                        <xsl:attribute name="localName"><xsl:value-of select="$FK2/references/@column"/></xsl:attribute>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:message terminate="yes">No relation found between
                            <xsl:value-of select="$PREC_TABLE"/>.<xsl:value-of select="$PREC_COLUMN"/> and
                            <xsl:value-of select="$TABLE"/>.<xsl:value-of select="$COLUMN"/>
                        </xsl:message>
                    </xsl:otherwise>
                </xsl:choose>
            </x:NameStep>
        </xsl:if>

        <!-- copy current NameStep -->
        <xsl:copy><xsl:copy-of select="@prefix"/>

            <!-- @localName -->
            <xsl:choose>
                <xsl:when test="following-sibling::x:NameStep[1]">
                    <xsl:variable name="FOLLOWING" select="following-sibling::x:NameStep[1]"/>
                    <xsl:variable name="PREC_TABLE" select="translate(@prefix, $lower, $UPPER)"/>
                    <xsl:variable name="PREC_COLUMN" select="translate(@localName, $lower, $UPPER)"/>
                    <xsl:variable name="TABLE" select="translate($FOLLOWING/@prefix, $lower, $UPPER)"/>
                    <xsl:variable name="COLUMN" select="translate($FOLLOWING/@linkWithPreceding, $lower, $UPPER)"/>
                    <xsl:variable name="FK1" select="$foreignKeys[@table=$PREC_TABLE and @column=$PREC_COLUMN]/references[@table=$TABLE]"/>
                    <xsl:variable name="FK2" select="$foreignKeys[@table=$TABLE and @column=$COLUMN and references[@table=$PREC_TABLE]]"/>
                    <xsl:choose>
                        <xsl:when test="$FK1">
                            <xsl:copy-of select="@localName"/>
                        </xsl:when>
                        <xsl:when test="$FK2">
                            <xsl:attribute name="localName"><xsl:value-of select="$FK2/references/@column"/></xsl:attribute>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:message terminate="yes">No relation found between
                                <xsl:value-of select="$PREC_TABLE"/>.<xsl:value-of select="$PREC_COLUMN"/> and
                                <xsl:value-of select="$TABLE"/>.<xsl:value-of select="$COLUMN"/>
                            </xsl:message>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:copy-of select="@localName"/>
                </xsl:otherwise>
            </xsl:choose>

            <!-- @pivot -->
            <xsl:choose>
                <xsl:when test="preceding-sibling::x:NameStep[1]">
                    <xsl:call-template name="CREATE_PIVOT">
                        <xsl:with-param name="PRECEDING" select="preceding-sibling::x:NameStep[1]"/>
                    </xsl:call-template>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:call-template name="CREATE_PIVOT">
                        <xsl:with-param name="PRECEDING" select="ancestor::x:NameStep[1][@prefix!=current()/@prefix]"/>
                    </xsl:call-template>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>

    <xsl:template name="CREATE_PIVOT">
        <xsl:param name="PRECEDING"/>
        <xsl:if test="$PRECEDING">
            <xsl:attribute name="pivot">
                <xsl:variable name="PREC_TABLE" select="translate($PRECEDING/@prefix, $lower, $UPPER)"/>
                <xsl:variable name="PREC_COLUMN" select="translate($PRECEDING/@localName, $lower, $UPPER)"/>
                <xsl:variable name="TABLE" select="translate(@prefix, $lower, $UPPER)"/>
                <xsl:variable name="COLUMN" select="translate(@linkWithPreceding, $lower, $UPPER)"/>
                <xsl:variable name="FK1" select="$foreignKeys[@table=$PREC_TABLE and @column=$PREC_COLUMN]/references[@table=$TABLE]"/>
                <xsl:variable name="FK2" select="$foreignKeys[@table=$TABLE and @column=$COLUMN and references[@table=$PREC_TABLE]]"/>
                <xsl:choose>
                    <xsl:when test="$FK1"><xsl:value-of select="$FK1/@column"/></xsl:when>
                    <xsl:when test="$FK2"><xsl:value-of select="$FK2/@column"/></xsl:when>
                    <xsl:otherwise>
                        <xsl:message terminate="yes">No relation found between
                            <xsl:value-of select="$PREC_TABLE"/>.<xsl:value-of select="$PREC_COLUMN"/> and
                            <xsl:value-of select="$TABLE"/>.<xsl:value-of select="$COLUMN"/>
                        </xsl:message>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:attribute>
        </xsl:if>
    </xsl:template>

    <xsl:template match="x:*">
        <xsl:copy><xsl:copy-of select="@*"/>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>