<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:cfg="http://software.in2p3.fr/lavoisier/config.xsd">
    <xsl:include href="/resource/xsl/head.xsl"/>
    <xsl:include href="/lavoisier/navbar"/>
    <xsl:output method="html" encoding="utf-8" indent="yes"/>

    <xsl:template match="/cfg:view">
        <html>
            <xsl:call-template name="head"/>
            <body>
                <xsl:call-template name="navbar"/>
                <div class="container">
                    <h2 class="page-title text-center"><xsl:value-of select="@name"/></h2>
                    <form id="form" method="get" class="form-horizontal" role="form">
                        <xsl:for-each select="cfg:argument">
                            <div class="form-group">
                                <label for="{@name}" class="col-sm-2 control-label"><xsl:value-of select="@name"/></label>
                                <div class="col-sm-10">
                                    <xsl:choose>
                                        <xsl:when test="contains(@pattern,'|') and not(contains(@pattern, '['))">
                                            <select name="{@name}" class="form-control">
                                                <xsl:call-template name="OPTIONS">
                                                    <xsl:with-param name="pattern" select="@pattern"/>
                                                    <xsl:with-param name="default" select="text()"/>
                                                </xsl:call-template>
                                            </select>
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <input type="text" name="{@name}" value="{text()}" class="span4"/>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </div>
                            </div>
                        </xsl:for-each>
                        <ul>
                            <xsl:if test="cfg:variable">
                                <li>
                                    <b>variables</b>
                                    <table class="table table-striped table-bordered">
                                        <xsl:apply-templates select="cfg:variable"/>
                                    </table>
                                </li>
                            </xsl:if>
                            <xsl:if test="cfg:namespace">
                                <li>
                                    <b>namespaces</b>
                                    <ul>
                                        <xsl:for-each select="cfg:namespace">
                                            <li>
                                                <b><xsl:value-of select="@prefix"/></b>
                                                <strong class="text-danger"> = </strong>
                                                <xsl:value-of select="@uri"/>
                                            </li>
                                        </xsl:for-each>
                                    </ul>
                                </li>
                            </xsl:if>
                            <xsl:for-each select="@authorized">
                                <li>
                                    <b><xsl:value-of select="name()"/></b>
                                    <strong class="text-danger"> = </strong>
                                    <div contenteditable="false" style="background-color: white;border: 2px solid #31b0d5;border-radius: 5px">
                                        <xsl:value-of select="."/>
                                    </div>
                                </li>
                            </xsl:for-each>
                            <xsl:for-each select="@timeout">
                                <li>
                                    <b><xsl:value-of select="name()"/></b>
                                    <strong class="text-danger"> = </strong>
                                    <xsl:value-of select="."/>
                                </li>
                            </xsl:for-each>
                            <xsl:for-each select="cfg:info">
                                <li>
                                    <b>info</b>
                                    <strong class="text-danger"> = </strong>
                                    <pre>
                                        <xsl:copy-of select="*"/>
                                    </pre>
                                </li>
                            </xsl:for-each>
                        </ul>

                        <xsl:apply-templates select="cfg:connector|cfg:serializer|cfg:validator|cfg:processors/cfg:processor|cfg:cache"/>
                        <xsl:apply-templates select="@post-processors"/>
                        <xsl:apply-templates select="cfg:pre-renderers|cfg:renderers/cfg:renderer"/>

                        <xsl:for-each select="cfg:processors[contains(cfg:processor/@disabled,'_stop_at_')]">
                            <b>Stop at:</b><br/>
                            <input type="radio" name="_stop_at_" value="0"/>
                            <xsl:value-of select="parent::cfg:view/cfg:connector/@type"/>
                            <br/>
                            <xsl:for-each select="cfg:processor[contains(@disabled,'_stop_at_')]">
                                <input type="radio" name="_stop_at_" value="{position()}"/>
                                <xsl:value-of select="@type"/>(<xsl:value-of select="@match"/>)
                                <br/>
                            </xsl:for-each>
                            <br/>
                        </xsl:for-each>

                        <input type="submit" value="Get data" onclick="get()" class="btn"/>
                        as
                        <select name="accept" id="accept">
                            <xsl:variable name="selected" select="cfg:info/cfg:accept/text()"/>
                            <option value="default">Default rendering</option>
                            <xsl:for-each select="jmx/domain/group/mbean/attribute[@name='Extensions']/value[1]">
                                <option value="{text()}">
                                    <xsl:if test="text() = $selected">
                                        <xsl:attribute name="selected">selected</xsl:attribute>
                                    </xsl:if>
                                    <xsl:value-of select="text()"/>
                                </option>
                            </xsl:for-each>
                        </select>
                        from
                        <input type="text" name="_window_start_" id="_window_start_" class="span1" value="1"/>
                        to
                        <input type="text" name="_window_end_" id="_window_end_" class="span1"/>
                        <input type="hidden" name="_window_" id="_window_"/>
                        <br/>

                        <input type="hidden" name="view" id="view" value="{@name}"/>
                        <input type="submit" value="Breakpoint" onclick="breakpoint()" class="btn"/>
                        on
                        <input type="text" name="_break_at_" id="_break_at_" value="/*/*" class="span4"/>
                        <br/>

                        <input type="submit" value="Get skeleton" onclick="skeleton()" class="btn"/>
                        <br/>

                        <input type="submit" value="Get events list" onclick="events()" class="btn"/>
                    </form>
                </div>
                <script language="Javascript">
function get() {
    document.getElementById('form').action='/lavoisier/<xsl:value-of select="@name"/>';
    if (document.getElementById('accept').value == 'default') {
        document.getElementById('accept').disabled=true;
    }
    document.getElementById('view').disabled=true;
    common();
}
function breakpoint() {
    document.getElementById('form').action='/lavoisier/breakpoint'+document.getElementById('_break_at_').value;
    document.getElementById('form').target='_blank';
    document.getElementById('accept').disabled=true;
    document.getElementById('view').disabled=false;
    common();
}
function skeleton() {
    document.getElementById('form').action='/lavoisier/skeleton/<xsl:value-of select="@name"/>';
    document.getElementById('accept').disabled=true;
    document.getElementById('view').disabled=true;
    common();
}
function events() {
    document.getElementById('form').action='/lavoisier/events/<xsl:value-of select="@name"/>';
    document.getElementById('accept').disabled=true;
    document.getElementById('view').disabled=true;
    common();
}
function common() {
    document.getElementById('_break_at_').disabled=true;
    document.getElementById('_window_start_').disabled=true;
    document.getElementById('_window_end_').disabled=true;
    if (document.getElementById('_window_end_').value != '') {
        document.getElementById('_window_').value = document.getElementById('_window_start_').value+'-'+document.getElementById('_window_end_').value;
    } else {
        document.getElementById('_window_').disabled=true;
    }
    $("#form input[type=text]").each(function() {
        if (this.value == '') {
            this.disabled = true;
        }
    })
}
                </script>
            </body>
        </html>
    </xsl:template>

    <xsl:template match="cfg:connector|cfg:serializer|cfg:validator|cfg:processor|cfg:cache|cfg:renderer">
        <div style="margin-top: 50px">
            <a id="{position()}"/>
            <xsl:for-each select="@match">
                <div contenteditable="false" style="background-color: white;border: 2px solid #31b0d5;border-radius: 5px">
                    <xsl:value-of select="."/>
                </div>
            </xsl:for-each>
            <table class="table table-striped table-bordered">
                <caption class="h3 bg-info">
                    <xsl:call-template name="CLASSNAME">
                        <xsl:with-param name="FQClass" select="@type"/>
                    </xsl:call-template>
                </caption>
                <xsl:apply-templates select="cfg:parameter | cfg:trigger"/>
            </table>
        </div>
    </xsl:template>

    <xsl:template match="cfg:trigger">
        <tr>
            <td class="h4 bg-info" width="100">
                <xsl:call-template name="CLASSNAME">
                    <xsl:with-param name="FQClass" select="@type"/>
                </xsl:call-template>
            </td>
            <td>
                <table class="table table-striped table-bordered">
                    <xsl:apply-templates select="cfg:parameter"/>
                </table>
            </td>
        </tr>
    </xsl:template>

    <xsl:template match="@post-processors">
        <table class="table table-striped table-bordered">
            <caption class="h3 bg-info">
                <a href="#">
                    <xsl:value-of select="."/>
                </a>
            </caption>
        </table>
    </xsl:template>

    <xsl:template match="cfg:pre-renderers">
        <table class="table table-striped table-bordered">
            <caption class="h3 bg-info">Pre-renderers</caption>
        </table>
    </xsl:template>

    <xsl:template match="cfg:parameter | cfg:entry | cfg:variable">
        <tr>
            <xsl:for-each select="@name | @key">
                <th width="100">
                    <xsl:value-of select="."/>
                </th>
            </xsl:for-each>
            <td>
                <xsl:choose>
                    <xsl:when test="@eval">
                        <div contenteditable="false" style="background-color: white;border: 2px solid #31b0d5;border-radius: 5px">
                            <xsl:value-of select="@eval"/>
                        </div>
                    </xsl:when>
                    <xsl:when test="cfg:entry">
                        <table class="table table-striped table-bordered" style="margin-bottom: 0px !important;">
                            <xsl:apply-templates select="cfg:entry"/>
                        </table>
                    </xsl:when>
                    <xsl:when test="text()='true' or text()='TRUE'">
                        <input type="checkbox" value="true"/>
                    </xsl:when>
                    <xsl:when test="text()='false' or text()='FALSE'">
                        <input type="checkbox"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <div contenteditable="false">
                            <xsl:value-of select="text()"/>
                        </div>
                    </xsl:otherwise>
                </xsl:choose>
            </td>
        </tr>
    </xsl:template>

    <xsl:template name="OPTIONS">
        <xsl:param name="pattern"/>
        <xsl:param name="default"/>
        <xsl:choose>
            <xsl:when test="contains($pattern, '|')">
                <xsl:variable name="current" select="substring-before($pattern, '|')"/>
                <option value="{$current}">
                    <xsl:if test="$current = $default">
                        <xsl:attribute name="selected">selected</xsl:attribute>
                    </xsl:if>
                    <xsl:value-of select="$current"/>
                </option>
                <xsl:call-template name="OPTIONS">
                    <xsl:with-param name="pattern" select="substring-after($pattern, '|')"/>
                    <xsl:with-param name="default" select="$default"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <option value="{$pattern}">
                    <xsl:if test="$pattern = $default">
                        <xsl:attribute name="selected">selected</xsl:attribute>
                    </xsl:if>
                    <xsl:value-of select="$pattern"/>
                </option>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="CLASSNAME">
        <xsl:param name="FQClass"/>
        <xsl:choose>
            <xsl:when test="contains($FQClass,'.')">
                <xsl:call-template name="CLASSNAME">
                    <xsl:with-param name="FQClass" select="substring-after($FQClass,'.')"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$FQClass"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>