<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:e="http://software.in2p3.fr/lavoisier/entries.xsd"
                xmlns:cfg="http://software.in2p3.fr/lavoisier/config.xsd">
    <xsl:output method="xml"/>
    <xsl:variable name="excludes" select="document('excludes')"/>

    <xsl:template match="/cfg:config"><entries xmlns="http://software.in2p3.fr/lavoisier/entries.xsd"><entry>digraph G {
        graph [rankdir="LR"];
        node [shape="record", fontname="Arial", fontsize=12];
        edge [];
        <xsl:variable name="views" select="cfg:view[not(cfg:info/cfg:category='Hidden' or cfg:info/cfg:category=$excludes/e:entries/e:entry)]"/>
        <xsl:apply-templates select="$views"/>
        <xsl:apply-templates select="$views" mode="EDGE"/>
}</entry></entries>
    </xsl:template>

    <xsl:template match="cfg:view[cfg:info/cfg:category]">
    subgraph "cluster_<xsl:value-of select="cfg:info/cfg:category/text()"/>" {
        style = "dashed";
        label = "<xsl:value-of select="cfg:info/cfg:category/text()"/>";
        <xsl:apply-templates select="self::cfg:view" mode="NODE"/>
    }
    </xsl:template>
    <xsl:template match="cfg:view[not(cfg:info/cfg:category)]">
        <xsl:apply-templates select="self::cfg:view" mode="NODE"/>
    </xsl:template>
    <xsl:template match="cfg:view" mode="NODE">
        <xsl:variable name="plugins" select="cfg:connector|cfg:serializer|cfg:validator|cfg:processors/cfg:processor|cfg:cache"/>
        "<xsl:value-of select="@name"/>" [
            id = "<xsl:apply-templates select="$plugins" mode="SYMBOL"/>_<xsl:value-of select="@name"/>",
            label = "&lt;v&gt; <xsl:value-of select="@name"/> <xsl:apply-templates select="$plugins" mode="RECORD"/>",
            href = "/lavoisier/debug/<xsl:value-of select="@name"/>"
        ];<xsl:text/>
    </xsl:template>
    <xsl:template match="cfg:connector|cfg:serializer|cfg:validator|cfg:processor|cfg:cache" mode="RECORD">
        <xsl:variable name="class_name">
            <xsl:call-template name="CLASS_NAME"><xsl:with-param name="string" select="@type"/></xsl:call-template>
        </xsl:variable>
        <xsl:variable name="suffix">
            <xsl:apply-templates select="." mode="SUFFIX"/>
        </xsl:variable>
        <xsl:text/> | &lt;f<xsl:value-of select="position()"/>&gt;<xsl:text/>
        <xsl:choose>
            <xsl:when test="contains($class_name, $suffix)">
                <xsl:value-of select="substring-before($class_name, $suffix)"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$class_name"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="cfg:view" mode="EDGE">
        <xsl:apply-templates select="cfg:argument" mode="EDGE"/>
        <xsl:apply-templates select="cfg:connector|cfg:serializer|cfg:validator|cfg:processors/cfg:processor|cfg:cache" mode="EDGE"/>
    </xsl:template>
    <xsl:template match="cfg:argument" mode="EDGE">
        <xsl:apply-templates select="@eval" mode="EDGE">
            <xsl:with-param name="target">"<xsl:value-of select="ancestor::cfg:view/@name"/>":v</xsl:with-param>
        </xsl:apply-templates>
    </xsl:template>
    <xsl:template match="cfg:connector|cfg:serializer|cfg:validator|cfg:processor|cfg:cache" mode="EDGE">
        <xsl:apply-templates select="cfg:parameter/@eval|cfg:parameter/cfg:entry/@eval|cfg:variable/@eval
                                    |cfg:parameter[@name='nodes']/text()" mode="EDGE">
            <xsl:with-param name="target">"<xsl:value-of select="ancestor::cfg:view/@name"/>":f<xsl:value-of select="position()"/></xsl:with-param>
        </xsl:apply-templates>
    </xsl:template>
    <xsl:template match="@eval | text()" mode="EDGE">
        <xsl:param name="target"/>
        <xsl:variable name="regexp">view(?:_path|_post|_path_post)?\s*\(\s*'(\w[\w\-\.]*)'</xsl:variable>
        <xsl:apply-templates select="regexp:match(., $regexp, 'g')" mode="EDGE" xmlns:regexp="xalan://exslt.org.regular_expressions">
            <xsl:with-param name="target" select="$target"/>
        </xsl:apply-templates>
    </xsl:template>
    <xsl:template match="token" mode="EDGE">
        <xsl:param name="target"/>
        <xsl:apply-templates select="/cfg:config/cfg:view[@name=current()/text()]" mode="SOURCE"/> -> <xsl:value-of select="$target"/>;<xsl:text/>
    </xsl:template>
    <xsl:template match="cfg:view" mode="SOURCE">
        "<xsl:value-of select="@name"/>":f<xsl:value-of select="count(cfg:connector|cfg:serializer|cfg:validator|cfg:processors/cfg:processor|cfg:cache)"/>
    </xsl:template>

    <xsl:template match="cfg:connector" mode="SUFFIX">Connector</xsl:template>
    <xsl:template match="cfg:serializer" mode="SUFFIX">Serializer</xsl:template>
    <xsl:template match="cfg:validator" mode="SUFFIX">Validator</xsl:template>
    <xsl:template match="cfg:processor" mode="SUFFIX">Processor</xsl:template>
    <xsl:template match="cfg:cache" mode="SUFFIX">Cache</xsl:template>

    <xsl:template match="cfg:connector" mode="SYMBOL">C</xsl:template>
    <xsl:template match="cfg:serializer" mode="SYMBOL">S</xsl:template>
    <xsl:template match="cfg:validator" mode="SYMBOL">V</xsl:template>
    <xsl:template match="cfg:processor" mode="SYMBOL">P</xsl:template>
    <xsl:template match="cfg:cache" mode="SYMBOL">K</xsl:template>

    <xsl:template name="CLASS_NAME">
        <xsl:param name="string"/>
        <xsl:choose>
            <xsl:when test="contains($string, '.')">
                <xsl:call-template name="CLASS_NAME">
                    <xsl:with-param name="string" select="substring-after($string, '.')"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$string"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>