<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:e="http://software.in2p3.fr/lavoisier/entries.xsd">
    <xsl:include href="/resource/xsl/head.xsl"/>
    <xsl:include href="/lavoisier/navbar"/>
    <xsl:output method="html" encoding="utf-8" indent="yes"/>
    <xsl:variable name="connector">#6cbddf</xsl:variable><!-- blue -->
    <xsl:variable name="serializer">#66c89d</xsl:variable><!-- green -->
    <xsl:variable name="validator">#e5a4f5</xsl:variable><!-- purple -->
    <xsl:variable name="processor">#efefef</xsl:variable><!-- grey -->
    <xsl:variable name="cache">#fbbf58</xsl:variable><!-- orange -->

    <xsl:template match="/">
        <html>
            <xsl:call-template name="head"/>
            <body>
                <xsl:call-template name="navbar"/>
                <div class="container">
                    <h2 class="page-title text-center">View Dependencies</h2>
                </div>
                <script type="text/vnd.graphviz" id="dot">
                    <xsl:value-of select="/e:entries/e:entry/text()"/>
                </script>
                <script src="/resource/js/viz.js"></script>
                <script><![CDATA[
function printStackTrace(s) {
    return "<pre>" + s.replace(/</g, "&lt;").replace(/>/g, "&gt;").replace(/\"/g, "&quot;") + "</pre>"
}
function toSVG() {
    try {
        return Viz(document.getElementById("dot").innerHTML, "svg");
    } catch(e) {
        return printStackTrace(e.toString());
    }
}
document.body.innerHTML += toSVG();
             ]]></script>

                <script src="/resource/js/jquery.min.js"></script>
                <script>
var color = new Object();
color["C"] = "<xsl:value-of select="$connector"/>";
color["S"] = "<xsl:value-of select="$serializer"/>";
color["V"] = "<xsl:value-of select="$validator"/>";
color["P"] = "<xsl:value-of select="$processor"/>";
color["K"] = "<xsl:value-of select="$cache"/>";

var plugin = new Object();
plugin["C"] = "Connector";
plugin["S"] = "Serializer";
plugin["V"] = "Validator";
plugin["P"] = "Processor";
plugin["K"] = "Cache";

createPluginBlocks();
createHyperLinks();

/**
 * Création des boites colorées derrière les textes
 **/
function createPluginBlocks(){
    $("g.node").each(function(){
        var sequence = $($(this).attr("id").split("_"))[0].split("");
        var i = 0;
        $(this).find("text:gt(0)").each(function(){
            var parentBox = $(this).prev("polyline").get(0).getBBox();
            if ( $(this).next("polyline").get(0) ) {
                var tmp = $(this).next("polyline").get(0).getBBox();
                parentBox.height = tmp.y - parentBox.y;
            } else {
                var tmp = $(this).prevAll("polygon").get(0).getBBox();
                parentBox.height = tmp.height + tmp.y - parentBox.y;
            }
            createBlock( $(this).get(0),  color[ sequence[i] ] , parentBox);
            i++;
        });
    });
}

/**
 * Fonction qui crée la boite derrière un élément texte
 **/
function createBlock( textElem , color , parentBox ) {
    var SVGRect = textElem.getBBox();
    var rect = document.createElementNS("http://www.w3.org/2000/svg", "rect");
    rect.setAttribute("class" , "block")
    rect.setAttribute("x", parentBox.x + 0.5);
    rect.setAttribute("y", parentBox.y + 0.5);
    rect.setAttribute("width", parentBox.width - 1);
    rect.setAttribute("height", parentBox.height - 1);
    rect.setAttribute("fill", color );
    $(rect).insertBefore(textElem);
}

/**
 * Décplacement du lien 'a' racine vers le 1er 'text' trouvé
 */
function createHyperLinks(){
    // On déplace tous les "g > a > *" à la racine de "g", donc "a" devient orphelin
    $("g.node").find("a").find("*").each(function(){
        $(this).clone().appendTo( $(this).closest("g")  );
        $(this).remove();
    });

    // On créer le lien sur le nom du plugin
    $("g.node").find("text:lt(1)").each(function(){
        $(this).css("font-style" , "italic" );
        $(this).wrap( $(this).siblings("a") );
        $(this).closest("a").attr("xlink:title" , "View");
    });

    // On crée des liens pour tous les aurtes blocks du plugin
    $("g.node").each(function(){
        var sequence = $($(this).attr("id").split("_"))[0].split("");
        var i = 0;
        $(this).find("text:gt(0)").each(function(){
            $(this).wrap( $(this).siblings("a") );
            $(this).closest("a").attr("xlink:href" , ($(this).closest("a").attr("xlink:href") + "#" + (i+1)) );
            $(this).closest("a").attr("xlink:title" , plugin[ sequence[i] ]);
            i++;
        });
    });

    // On supprime le g > a orphelin
    $("g.node").each(function(){
        $(this).find("a").first().remove();
    });
}

// *** Intéractions souris
$("a").mouseover(function(){
    $(this).css('opacity' , 0.75);
    $("#debug").css("display" , "block");
    $("#debug").html( $(this).attr("xlink:href") );
});

$("a").mouseleave(function(){
    $(this).css('opacity' , 1);
    $("#debug").css("display" , "none");
    $("#debug").html("");
});

$("a").mousemove(function( event ) {
    $("#debug").css("position" , "absolute");
    $("#debug").css("z-index" , 99999);
    $("#debug").css("top" , event.pageY - 20);
    $("#debug").css("left" , event.pageX + 20 );
});
                </script>
                <div style="margin: 20px">
                    <ul class="list-inline">
                        <li><u>Legend</u>:</li>
                        <li><span style="background-color:{$connector}">Connector</span></li>
                        <li><span style="background-color:{$serializer}">Serializer</span></li>
                        <li><span style="background-color:{$validator}">Validator</span></li>
                        <li><span style="background-color:{$processor}">Processor</span></li>
                        <li><span style="background-color:{$cache}">Cache</span></li>
                    </ul>
                </div>
            </body>
        </html>
    </xsl:template>
</xsl:stylesheet>