<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:include href="/resource/xsl/head.xsl"/>
    <xsl:include href="/lavoisier/navbar"/>
    <xsl:output method="html"/>
    <xsl:key name="xmlns" match="//*" use="concat(substring-before(name(),':'),'=',namespace-uri())"/>

    <xsl:template match="/">
        <html>
            <xsl:call-template name="head"/>
            <body>
                <xsl:call-template name="navbar"/>
                <div class="container">
                    <h2 class="page-title text-center">Click on XML to get XPath</h2>
                    <pre>
                        <xsl:apply-templates>
                            <xsl:with-param name="indent"><xsl:text>
</xsl:text></xsl:with-param>
                            <xsl:with-param name="namespaces"/>
                        </xsl:apply-templates>
                    </pre>
                </div>

                <!-- modal window -->
                <script language="JavaScript">
$(document).ready(function(){
    $("a[data-xpath]").click(function(){
        var xpath = $(this).attr("data-xpath");
        $('.modal-body>div').text(xpath);
        $('.bs-example-modal-sm').modal('toggle');
    });
});
                </script>
                <div class="modal fade bs-example-modal-sm" tabindex="-1" role="dialog" aria-labelledby="myModal" aria-hidden="true">
                    <div class="modal-dialog modal-sm">
                        <div class="modal-content">
                            <div class="modal-header">
                                <button aria-hidden="true" data-dismiss="modal" class="close" type="button">X</button>
                                <h3 id="myModal" class="modal-title">Use right-click to copy...</h3>
                            </div>
                            <div class="modal-body">
                                <h4>XPath</h4>
                                <div>...</div>
                                <h4>Namespaces</h4>
                                <ul>
                                    <xsl:for-each select="//*[namespace-uri()!='']
                                                             [generate-id()=generate-id(key('xmlns',
                                            concat(substring-before(name(),':'),'=',namespace-uri()))[1])]">
                                        <li>
                                            <xsl:text>xmlns:</xsl:text>
                                            <xsl:choose>
                                                <xsl:when test="contains(name(),':')">
                                                    <xsl:value-of select="substring-before(name(),':')"/>
                                                </xsl:when>
                                                <xsl:otherwise>
                                                    <xsl:text>ns_</xsl:text>
                                                    <xsl:value-of select="count(ancestor::*)"/>
                                                </xsl:otherwise>
                                            </xsl:choose>
                                            <xsl:text>="</xsl:text>
                                            <xsl:value-of select="namespace-uri()"/>
                                            <xsl:text>"</xsl:text>
                                        </li>
                                    </xsl:for-each>
                                </ul>
                            </div>
                        </div>
                    </div>
                </div>
            </body>
        </html>
    </xsl:template>

    <xsl:template match="*">
        <xsl:param name="indent"/>
        <xsl:param name="namespaces"/>
        <xsl:param name="xpath"/>

        <xsl:variable name="prefix">
            <xsl:choose>
                <xsl:when test="namespace-uri()=''"/>
                <xsl:when test="contains(name(),':')">
                    <xsl:value-of select="substring-before(name(),':')"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:for-each select="key('xmlns', concat(substring-before(name(),':'),'=',namespace-uri()))[1]">
                        <xsl:text>ns_</xsl:text>
                        <xsl:value-of select="count(ancestor::*)"/>
                    </xsl:for-each>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="ns" select="concat('|',$prefix,'=',namespace-uri(),'|')"/>
        <xsl:variable name="newNamespaces">
            <xsl:value-of select="$namespaces"/>
            <xsl:if test="not(contains($namespaces, $ns))"><xsl:value-of select="$ns"/></xsl:if>
        </xsl:variable>
        <xsl:variable name="newXPath">
            <xsl:value-of select="concat($xpath,'/')"/>
            <xsl:if test="$prefix!=''"><xsl:value-of select="$prefix"/>:</xsl:if>
            <xsl:value-of select="local-name()"/>
        </xsl:variable>

        <xsl:if test="string-length($indent)>1"><xsl:value-of select="$indent"/></xsl:if>&lt;<xsl:text/>
        <xsl:call-template name="tag">
            <xsl:with-param name="xpath" select="$newXPath"/>
        </xsl:call-template>
        <xsl:apply-templates select="@*">
            <xsl:with-param name="xpath" select="$newXPath"/>
        </xsl:apply-templates>

        <xsl:if test="namespace-uri()!='' and not(contains($namespaces, $ns))">
            <xsl:text> xmlns</xsl:text>
            <xsl:if test="contains(name(),':')">:<xsl:value-of select="substring-before(name(),':')"/></xsl:if>
            <xsl:text/>="<xsl:value-of select="namespace-uri(.)"/>"<xsl:text/>
        </xsl:if>

        <xsl:choose>
            <xsl:when test="*|comment()">
                <xsl:text>&gt;</xsl:text>
                <xsl:apply-templates>
                    <xsl:with-param name="indent" select="concat($indent,'    ')"/>
                    <xsl:with-param name="namespaces" select="$newNamespaces"/>
                    <xsl:with-param name="xpath" select="$newXPath"/>
                </xsl:apply-templates>
                <xsl:value-of select="$indent"/>&lt;/<xsl:text/>
                <xsl:call-template name="tag">
                    <xsl:with-param name="xpath" select="$newXPath"/>
                </xsl:call-template>
                <xsl:text>&gt;</xsl:text>
            </xsl:when>
            <xsl:when test="text()">
                <xsl:text>&gt;</xsl:text>
                <xsl:apply-templates select="text()">
                    <xsl:with-param name="xpath" select="$newXPath"/>
                </xsl:apply-templates>
                <xsl:text>&lt;/</xsl:text>
                <xsl:call-template name="tag">
                    <xsl:with-param name="xpath" select="$newXPath"/>
                </xsl:call-template>
                <xsl:text>&gt;</xsl:text>
            </xsl:when>
            <xsl:otherwise>
                <xsl:text>/&gt;</xsl:text>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template match="@*">
        <xsl:param name="xpath"/>
        <xsl:text> </xsl:text>
        <xsl:call-template name="attribute_name">
            <xsl:with-param name="xpath" select="$xpath"/>
        </xsl:call-template>
        <xsl:text/>="<xsl:call-template name="attribute_value"/>"<xsl:text/>
    </xsl:template>

    <xsl:template match="comment()">
        <xsl:param name="xpath"/>
        <xsl:param name="indent"/>
        <xsl:value-of select="$indent"/>
        <a href="#" data-xpath="{$xpath}/comment()">
            <font color="#385">&lt;!--<xsl:value-of select="."/>--&gt;</font>
        </a>
    </xsl:template>
    <xsl:template match="text()">
        <xsl:param name="xpath"/>
        <a href="#" data-xpath="{$xpath}/text()">
            <font color="#000"><xsl:value-of select="normalize-space(.)"/></font>
        </a>
    </xsl:template>
    <xsl:template name="tag">
        <xsl:param name="xpath"/>
        <a href="#" data-xpath="{$xpath}">
            <b><font color="#a05"><xsl:value-of select="name()"/></font></b>
        </a>
    </xsl:template>
    <xsl:template name="attribute_name">
        <xsl:param name="xpath"/>
        <a href="#" data-xpath="{$xpath}/@{name()}">
            <font color="#b48"><xsl:value-of select="local-name()"/></font>
        </a>
    </xsl:template>
    <xsl:template name="attribute_value">
        <i><font color="#46a"><xsl:value-of select="."/></font></i>
    </xsl:template>
</xsl:stylesheet>